package usercontract

import (
	"encoding/json"
	"strconv"
	"testing"

	"github.com/golang/mock/gomock"

	"git.huawei.com/poissonsearch/wienerchain/contract/docker-container/contract-go/contractapi"
)

var (
	key = "marble01"

	value = &marble{
		ObjectType: "marble",
		Name:       "marble01",
		Color:      "red",
		Size:       10,
		Owner:      "zhangsan",
	}

	startKey, endKey = "marble01", "marble03"
)

func TestInsertMarble(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	stub := contractapi.NewMockContractStub(ctrl)
	// 打桩，指定所调用方法中使用到的接口的入参和返回值

	stub.EXPECT().FuncName().Return("insertMarble")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key), []byte(value.Owner), []byte(value.Color), []byte(strconv.Itoa(value.Size))})
	stub.EXPECT().PutKVCommon(key, value).Return(nil)
	// 方法调用
	invoke(t, stub)
}

func TestQueryMarble(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	stub := contractapi.NewMockContractStub(ctrl)
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	stub.EXPECT().FuncName().Return("queryMarble")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key)})
	marshal, err := json.Marshal(value)
	stub.EXPECT().GetKV(key).Return(marshal, err)
	// 方法调用
	invoke(t, stub)
}

func TestDeleteMarble(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	stub := contractapi.NewMockContractStub(ctrl)
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	stub.EXPECT().FuncName().Return("deleteMarble")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key)})
	marshal, err := json.Marshal(value)
	stub.EXPECT().GetKV(key).Return(marshal, err)
	stub.EXPECT().DelKV(key).Return(nil)
	// 方法调用
	invoke(t, stub)
}

func TestQueryMarbleKeyHistory(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	historyIterator := contractapi.NewMockHistoryIterator(ctrl)
	historyIterator.EXPECT().Next().Return(true)
	marshal, err := json.Marshal(value)
	if err != nil {
		t.Fatalf("invoke failed, err is: %v", err)
	}
	historyIterator.EXPECT().Value().Return(marshal)
	historyIterator.EXPECT().Close().Return()
	historyIterator.EXPECT().Version().Return(uint64(2), int32(0))
	historyIterator.EXPECT().TxHash().Return([]byte("xxxxxxxxxxxxxxxxxxxxxx"))
	historyIterator.EXPECT().IsDeleted().Return(false)
	historyIterator.EXPECT().Timestamp().Return(uint64(1636440016000000))
	historyIterator.EXPECT().Next().Return(false)

	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("queryMarbleKeyHistory")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key)})
	stub.EXPECT().GetKeyHistoryIterator(startKey).Return(historyIterator, nil)
	// 方法调用
	invoke(t, stub)
}

func TestQueryMarbleInRange(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	iterator := contractapi.NewMockIterator(ctrl)
	iterator.EXPECT().Next().Return(true)
	iterator.EXPECT().Key().Return(key)
	marshal, err := json.Marshal(value)
	if err != nil {
		t.Fatalf("invoke failed, err is: %v", err)
	}
	iterator.EXPECT().Value().Return(marshal)
	iterator.EXPECT().Close().Return()
	iterator.EXPECT().Next().Return(false)
	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("queryMarbleInRange")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(startKey), []byte(endKey)})
	stub.EXPECT().GetIterator(startKey, endKey).Return(iterator, nil)
	// 方法调用
	invoke(t, stub)
}

func TestInsertMarbleAndIndex(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	stub := contractapi.NewMockContractStub(ctrl)
	// 打桩，指定所调用方法中使用到的接口的入参和返回值

	stub.EXPECT().FuncName().Return("insertMarbleAndIndex")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key), []byte(value.Owner), []byte(value.Color), []byte(strconv.Itoa(value.Size))})
	stub.EXPECT().PutKVCommon(key, value).Return(nil)
	stub.EXPECT().SaveComIndex(indexName, []string{value.Color, strconv.Itoa(value.Size)}, key).Return(nil)
	// 方法调用
	invoke(t, stub)
}

func TestQueryMarblesByIndex(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	stub := contractapi.NewMockContractStub(ctrl)
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	iterator := contractapi.NewMockIterator(ctrl)
	iterator.EXPECT().Next().Return(true)
	iterator.EXPECT().Key().Return(key)
	marshal, err := json.Marshal(value)
	if err != nil {
		t.Fatalf("invoke failed, err is: %v", err)
	}
	iterator.EXPECT().Value().Return(marshal)
	iterator.EXPECT().Close().Return()
	iterator.EXPECT().Next().Return(false)

	stub.EXPECT().FuncName().Return("queryMarblesByIndex")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(value.Color), []byte(strconv.Itoa(value.Size))})
	stub.EXPECT().GetKVByComIndex(indexName, []string{value.Color, strconv.Itoa(value.Size)}).Return(iterator, nil)
	// 方法调用
	invoke(t, stub)
}

func TestTransferMarblesByIndex(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	iterator := contractapi.NewMockIterator(ctrl)
	iterator.EXPECT().Next().Return(true)
	iterator.EXPECT().Key().Return(key)
	marshal, err := json.Marshal(value)
	if err != nil {
		t.Fatalf("invoke failed, err is: %v", err)
	}
	iterator.EXPECT().Value().Return(marshal)
	iterator.EXPECT().Close().Return()
	iterator.EXPECT().Next().Return(false)
	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("transferMarblesByIndex")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(value.Color), []byte(strconv.Itoa(value.Size)), []byte("lisi")})
	stub.EXPECT().GetKVByComIndex(indexName, []string{value.Color, strconv.Itoa(value.Size)}).Return(iterator, nil)
	// 更换拥有者
	marbleToTransfer := marble{}
	err = json.Unmarshal(marshal, &marbleToTransfer)
	if err != nil {
		t.Fatalf("invoke failed, err is: %v", err)
	}
	marbleToTransfer.Owner = "lisi"
	stub.EXPECT().PutKVCommon(key, marbleToTransfer).Return(nil)
	// 方法调用
	invoke(t, stub)
}

func TestDeleteMarbleAndIndex(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("deleteMarbleAndIndex")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key)})
	marshal, err := json.Marshal(value)
	if err != nil {
		t.Fatalf("invoke failed, err is: %v", err)
	}
	stub.EXPECT().GetKV(key).Return(marshal, nil)
	stub.EXPECT().DelKV(key).Return(nil)
	stub.EXPECT().DelComIndexOneRow(indexName, []string{value.Color, strconv.Itoa(value.Size)}, key).Return(nil)
	// 方法调用
	invoke(t, stub)
}

func invoke(t *testing.T, stub *contractapi.MockContractStub) {
	var contract = NewSmartContract()
	invokeRes, err := contract.Invoke(stub)
	if err != nil {
		t.Fatalf("invoke failed, err is: %v", err)
	}
	t.Logf("invoke success: %s\n", string(invokeRes))
}
